#include <stdio.h>

#include <VolumeRoster.h>
#include <Directory.h>
#include <Node.h>
#include <NodeInfo.h>
#include <Alert.h>

#include "Colors.h"
#include "SafeTime.h"

#include "SeekerWindow.h"
#include "SFileView.h"

#include "SFileInfo.h"

SFileInfoView::SFileInfoView(SFileInfoWindow *infoWindow,
							  const SFile *file,
							  BRect frame,
					   		  const char *name=NULL,
					   		  uint32 resizingMode=B_FOLLOW_ALL,
					   		  uint32 flags=B_WILL_DRAW)
 : BView(frame, name, resizingMode, flags),
   iInfoWindow(infoWindow),
   iFile(0),
   iTextOffset(0.0)
{
	iFile=new SFile(file); 

	/* set font height */
	font_height FontAttributes;
	be_plain_font->GetHeight(&FontAttributes);
	
	iTextOffset=ceil(FontAttributes.ascent) + 
				  ceil(FontAttributes.descent) +
				  ceil(FontAttributes.leading);
	iTextOffset *=1.5;
}

SFileInfoView::~SFileInfoView()
{
	if(iFile) delete iFile;
}

/* implemented Draw function */
void SFileInfoView::Draw(BRect rect)
{
	/* draw shadow */
	BRect shadowRect;
	shadowRect.left=0.0;
	shadowRect.top=0;
	shadowRect.right=60.0;
	shadowRect.bottom=Bounds().Height();
		
	SetDrawingMode(B_OP_COPY);
	SetHighColor(BeInactiveControlGrey);
	FillRect(shadowRect);

	/* draw icon */
	BRect bitmapRect;
	bitmapRect.left=10.0;
	bitmapRect.right=bitmapRect.left + 31.0;
	bitmapRect.top=10.0;
	bitmapRect.bottom=bitmapRect.top + 31.0;
	SetDrawingMode(B_OP_OVER);
	DrawBitmap(iFile->LargeIcon(), BRect(0.0, 0.0, 31.0, 31.0), bitmapRect);
	SetDrawingMode(B_OP_COPY);

	/* draw name */
	BFont font;
	GetFont(&font);
	font.SetSize(14.0);
	SetFont(&font);
	MovePenTo(shadowRect.right + 10.0, bitmapRect.bottom);
	SetHighColor(Black);
	DrawString(iFile->Name());
	SetFont(be_plain_font);	

	/* draw information strings */
	font_height fontHeight;
	char strg[400];
	float x=0.0, y=50.0;

	GetFont(&font);
	font.GetHeight(&fontHeight);
 	SetDrawingMode(B_OP_OVER);
	
	/* SIZE */
	
	/* label */
	if(iFile->IsVolume()) {
		strcpy(strg, "Capacity");
	} else {
		strcpy(strg, "Size");
	}
	x=50.0 - font.StringWidth(strg);
	y +=iTextOffset; 
	MovePenTo(x, y);
	SetHighColor(Red);
	DrawString(strg);

	/* data */
	x=shadowRect.right + 10.0;
	MovePenTo(x, y);
	if(iFile->IsVolume()) {
		char capacityStrg[20], usedStrg[20], freeStrg[20];
		iFile->CapacityDesc(capacityStrg);
		iFile->UsedSpaceDesc(usedStrg);
		iFile->FreeSpaceDesc(freeStrg);
		sprintf(strg, "%s (%s used -- %s free)", capacityStrg, usedStrg, freeStrg);
	} else {
		iFile->SizeDesc(strg);
	}
	SetHighColor(Black);
	DrawString(strg);

	/* CREATED */
	
	/* label */
	strcpy(strg, "Created");
	x=50.0 - font.StringWidth(strg);
	y +=iTextOffset; 
	MovePenTo(x, y);
	SetHighColor(Red);
	DrawString(strg);

	/* data */
	x=shadowRect.right + 10.0;
	MovePenTo(x, y);
	iFile->CreatedTmDesc(strg);
	SetHighColor(Black);
	DrawString(strg);

	/* MODIFIED */
	
	/* label */	
	strcpy(strg, "Modified");
	x=50.0 - font.StringWidth(strg);
	y +=iTextOffset; 
	MovePenTo(x, y);
	SetHighColor(Red);
	DrawString(strg);

	/* data */
	x=shadowRect.right + 10.0;
	MovePenTo(x, y);
	iFile->ModifiedTmDesc(strg);
	SetHighColor(Black);
	DrawString(strg);

	/* KIND */
	
	/* label */	
	strcpy(strg, "Kind");
	x=50.0 - font.StringWidth(strg);
	y +=iTextOffset; 
	MovePenTo(x, y);
	SetHighColor(Red);
	DrawString(strg);

	/* data */
	x=shadowRect.right + 10.0;
	MovePenTo(x, y);
	SetHighColor(Black);
	DrawString(iFile->MimeDesc());

	/* PATH */
	
	/* label */	
	strcpy(strg, "Path");
	x=50.0 - font.StringWidth(strg);
	y +=iTextOffset; 
	MovePenTo(x, y);
	SetHighColor(Red);
	DrawString(strg);

	/* data */
	x=shadowRect.right + 10.0;
	MovePenTo(x, y);
	SetHighColor(Black);
	DrawString(iFile->PathDesc());
}

SFileInfoWindow::SFileInfoWindow(SeekerWindow *mainWindow,
								  const SFile *file)
 : BWindow(windowFrame(), "", B_FLOATING_WINDOW,
			B_WILL_DRAW),
   iMainWindow(mainWindow),
   iFile(0),
   iFileInfoView(0)
{
	iFile=new SFile(file);

	/* set window title */
	char strg[B_FILE_NAME_LENGTH + 10];
	sprintf(strg, "%s info", iFile->Name());
	SetTitle(strg);

	/* create view */
	iFileInfoView=new SFileInfoView(this, iFile, Bounds(),
									   "FileInfoWindow::FileInfoView");
	AddChild(iFileInfoView);
}

SFileInfoWindow::~SFileInfoWindow()
{
	if(iFile) delete iFile;

	/* remove from subset */
	BMessage *message=new BMessage(MSG_PIONEER_REMOVE_FROM_SUBSET);
	message->AddPointer("window", this);
	iMainWindow->Looper()->PostMessage(message);
	delete message;
}
