#ifndef _PFILEVIEW_H_
#define _PFILEVIEW_H_

#include <Bitmap.h>
#include <Entry.h>
#include <Path.h>
#include <OS.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "CLVListItem.h"
#include "ColumnListView.h"

class SFileView;

class SFileItem : public CLVListItem
{
	public:

		SFileItem(SFileView *FileView, const SFile *file);
		~SFileItem();
		
		/* custom paint */
		virtual void DrawItemColumn(BView *owner, BRect itemColumnRect, 
									 int32 columnIndex, bool complete);
		
		/* custom update */
		virtual void Update(BView *owner, const BFont *font);
		
		/* get file */
		SFile* File() {
			return iFile;
		}

	private:

		SFileView*  iFileView;
		SFile*		iFile;
		
		float 		iTextOffset;
		font_height iFontAttr;
};

class SFileView : public ColumnListView
{
	public:
	
		SFileView(BRect bounds, char *name);
		~SFileView();
		
		/* implemented MouseDown handler */
		virtual void MouseDown(BPoint point);
		virtual void KeyDown(const char *bytes,int32 numbytes);
		
		/* return container view */
		CLVContainerView* View() {
			return iContainerView;
		}

		/* set columns */
		void SetColumns(bool useName,
						 bool useSize,
						 bool useModified,
						 bool useCreated,
						 bool useKind,
						 bool usePath 		);

		/* refresh filelist */
		void Refresh(BMessage *message);
		
		/* clear all items */
		void Clear() {
			Refresh(NULL);
		}
		
		/* current selections */
		void CurrentSelections(SFileList *fileList);	
		
		/* open file or directory */
		void Open(BMessage *message);
		
		/* select all entries */
		void SelectAll();
		
		/* invert selection */
		void InvertSelection();

		/* node monitor: scan for created */
		void ScanForCreated(const SFile *file);

		/* node monitor: scan for moved */
		void ScanForMoved(const SFile *fromFile, const SFile *toFile);

		/* node monitor: scan for removed */
		void ScanAllForRemoved();
		void SpawnTracker();

	protected:
	
		/* build columns */
		void BuildColumns();

		/* sorting function */
		static int Compare(const CLVListItem* aItem1, const CLVListItem* aItem2, int32 sortKey);

		/* shared thread information */
		thread_id			iScanThreadId;
		sem_id				iScanThreadActive;
		sem_id				iScanThreadTerminate;
		entry_ref			iScanEntryRef;

		/* threaded file scan function */
		static int32 ScanThread(void *dataPtr);

		/* request scan termination and wait for it to stop */
		void RequestScanTerminate();

	private:
	
		CLVContainerView*	iContainerView;
		CLVListItem *iTreeItem;
				
		bool				iUseName,
							iUseSize,
							iUseModified,
							iUseCreated,
							iUseKind,
							iUsePath;

		CLVColumn			*iColIcon,
							*iColName,
							*iColSize,
							*iColModified,
							*iColCreated,
							*iColKind,
							*iColPath;

		bool	iFileScan;
};

#endif
