#ifndef _PFILEWORKER_H_
#define _PFILEWORKER_H_

#include <List.h>
#include <Window.h>
#include <StatusBar.h>
#include <StringView.h>
#include <TextControl.h>
#include <Button.h>

class SeekerWindow;

#define P_OP_NONE		0
#define P_OP_COPY		1
#define P_OP_CUT		2
#define P_OP_PASTE		3
#define P_OP_PASTE_LINK	4
#define P_OP_REMOVE		5

class SFileWorker
{
	public:
		SFileWorker( SeekerWindow *mainWindow );
		~SFileWorker( );

		/* single stage operations */
		void NewFolder( const SFile *parentFolder );
		void GetInfo( const SFile *file );
		void EditName( const SFile *file );
		void MoveToTrash( const SFile *srcFolder, const SFileList *fileList );
		void EmptyTrash( void );

		/* add files to the copy list */
		void Copy( const SFile* srcFolder, const SFileList *fileList );
		void Cut( const SFile* srcFolder, const SFileList *fileList );

		/* start a file operation */
		void Paste( const SFile *folder );
		void PasteLink( const SFile *folder );

	private:

		SeekerWindow*	iWindow;

		int				iFileOp;
		SFile*			iSrcFolder;		
		BList*			iFileList;		
};

class PNewFolderWindow : public BWindow
{
	public:
	
		PNewFolderWindow( SeekerWindow *mainWindow,
					 	  const SFile *parentFile );
		~PNewFolderWindow( );
		
		virtual void MessageReceived( BMessage* message );
		
	protected:
	
		static const BRect windowFrame ( ) {
			return BRect( 50, 50, 350, 200 );
		}
		
		void CreateFolder( );

	private:
	
		SeekerWindow*	iMainWindow;
		SFile*			iParentFile;

		BView*			iView;

		BTextControl*	iParentTextControl;
		BTextControl*	iNameTextControl;
		
		BButton*		iOkButton;
		BButton*		iCancelButton;
};

class PRenameWindow : public BWindow
{
	public:
	
		PRenameWindow( SeekerWindow *mainWindow,
					   const SFile *file );
		~PRenameWindow( );
		
		virtual void MessageReceived( BMessage* message );
		
	protected:
	
		static const BRect windowFrame ( ) {
			return BRect( 50, 50, 350, 200 );
		}
		
		void RenameFile( );

	private:
	
		SeekerWindow*	iMainWindow;
		SFile*			iFile;

		BView*			iView;

		BTextControl*	iOldNameTextControl;
		BTextControl*	iNewNameTextControl;
		
		BButton*		iOkButton;
		BButton*		iCancelButton;
};

class PCopyWindow : public BWindow
{
	public:
		
		PCopyWindow( SeekerWindow *mainWindow, 
					 SFile *srcFolder, BList *fileList, 
					 const SFile *destFolder, int copyOp, int pasteOp );
		~PCopyWindow( );
		
		virtual void MessageReceived( BMessage* message );

		virtual void Show( void );
		
	protected:
	
		static const BRect windowFrame ( ) {
			return BRect( 50, 50, 350, 100 );
		}

		/* shared thread information */
		thread_id			iCopyThreadId;
		sem_id				iCopyThreadActive;
		sem_id				iCopyThreadTerminate;

		/* threaded file scan function */
		static int32 CopyThread( void *dataPtr );

		/* request copy termination and wait for it to stop */
		void RequestCopyTerminate( );
		
	private:
	
		SeekerWindow*	iMainWindow;

		BView*			iView;

		BStatusBar*		iStatusBar;
		BButton*		iCancelButton;	

		SFile*			iSrcFolder;
		BList*			iFileList;
		SFile*			iDestFolder;
		int				iCopyOp;
		int				iPasteOp;
	
};

#endif
