#ifndef _PTREEVIEW_H_
#define _PTREEVIEW_H_

#include <Bitmap.h>
#include <Entry.h>
#include <VolumeRoster.h>

#include "CLVListItem.h"
#include "ColumnListView.h"

#include "SFile.h"

class STreeView;

class STreeItem : public CLVListItem
{
	public:

		STreeItem(STreeView *treeView, const SFile *file, bool expand=false);
		~STreeItem();
		
		virtual void DrawItemColumn(BView *owner, BRect itemColumnRect, 
									 int32 columnIndex, bool complete);
		virtual void Update(BView *owner, const BFont *font);
		
		// expand the item and select a specific directory
		void ExpandAndSelect(SFile *file);

		bool IsScanning();
		bool IsScanned();
		void ResetIcon();
		bool IsVolume()
		{
			if (iFile) return iFile->IsVolume();
			else return false;
		}
		SFile* File() {	return iFile; }

	protected:

		sem_id				iScanThreadActive;
		sem_id				iScanThreadTerminate;

		static int32 ScanThread(void *dataPtr);
		void SetAutoSelectOnScanDir(const char *name);
		void RequestScanTerminate();
	
	private:

		STreeView*  iTreeView;
		SFile*		iFile;

		char 		iAutoSelectDir[B_FILE_NAME_LENGTH];
		float 		iTextOffset;
		font_height iFontAttr;
		bool		iTreeScan;
		bool		iFileScan;
};

class STreeView : public ColumnListView
{
	public:
	
		STreeView(BRect bounds, char *name);
		~STreeView();

		virtual void MouseDown(BPoint point);
		CLVContainerView* View() {	return iContainerView; }

		void Refresh();
		void CurrentSelections(SFileList *fileList);	
		void SpawnTracker();
		
		void ExpandAndSelect(BMessage *message);

		void ScanForCreated(const SFile *file);
		void ScanForMoved(const SFile *fromFile, const SFile *toFile);
		void ScanAllForRemoved();
		void ScanUpdateTrashIcon();
		virtual void KeyDown(const char *bytes,int32 numbytes);
		
	protected:
		void BuildColumns();
		static int Compare(const CLVListItem* aItem1, const CLVListItem* aItem2, int32 sortKey);

	private:
	
		CLVContainerView*	iContainerView;
		BVolumeRoster *volumeRoster;
};

#endif