#include <stdio.h>
#include <Application.h>
#include <Menu.h>
#include <MenuItem.h>
#include <NodeMonitor.h>
#include <VolumeRoster.h>
#include <Volume.h>
#include <Messenger.h>

#include "SeekerApp.h"
#include "SeekerWindow.h"

#include "SFileInfo.h"
#include "SplitterView.h"

#define WINDOW_TITLE	"Seeker"

void NewItem(BMenu *menu, BMenuItem **item, char *label, uint16 id, uint16 shortcut=0)
{
	if(shortcut) {
		*item=new BMenuItem(label, new BMessage(id), (char) shortcut);
	} else {
		*item=new BMenuItem(label, new BMessage(id));
	}
	menu->AddItem(*item);
} 

SeekerWindow::SeekerWindow(SeekerApp *app)
 : BWindow(windowFrame(), WINDOW_TITLE, B_DOCUMENT_WINDOW,
			B_WILL_DRAW),
   iSplitterView(0),
   iTreeView(0),
   iFileView(0),
   iNodeMonitor(0),
   iFileWorker(0)
{
	this->iApp=app;

	/* create menus */
	BuildMenuBar();
	
	BRect bounds=this->Bounds();

	/* modify bounds to accomodate menu and scrollbars */
	bounds.top +=iMenuBar->Bounds().Height() + 1.0;
	iSplitterView=new CSplitterView(bounds, "SeekerWindow::SplitterView");
	AddChild(iSplitterView);	

	/* create tree-view */
	BuildTreeView();
	
	/* create file-view */
	BuildFileView();

	/* add node monitor handler to our looper list */
	iNodeMonitor=new SNodeMonitor(iTreeView, iFileView);
	Looper()->AddHandler(iNodeMonitor); 
	
	/* add file worker */
	iFileWorker=new SFileWorker(this);
	
	/* initial load of directories/files */
	iTreeView->Refresh();	
}

SeekerWindow::~SeekerWindow()
{
	if(iNodeMonitor) delete iNodeMonitor;
	if(iFileWorker) delete iFileWorker;
}

void SeekerWindow::BuildMenuBar()
{	
	
	iMenuBar=new BMenuBar(this->Bounds(), "SeekerWindow::MenuBar");

	iFileMenu=new BMenu("File");
		NewItem(iFileMenu, &iFileNewFolderItem, "New Folder", MID_FILE_NEW_FOLDER, 'N');
		iFileMenu->AddSeparatorItem();

		NewItem(iFileMenu, &iFileOpenItem, "Open", MID_FILE_OPEN, 'O');
		NewItem(iFileMenu, &iFileGetInfoItem, "Get info", MID_FILE_GET_INFO, 'I');
		NewItem(iFileMenu, &iFileEditNameItem, "Rename", MID_FILE_EDIT_NAME, 'E');
		NewItem(iFileMenu, &iFileMoveToTrashItem, "Send to Trash", MID_FILE_MOVE_TO_TRASH, 'T');
		NewItem(iFileMenu, &iFileEmptyTrashItem, "Empty trash", MID_FILE_EMPTY_TRASH);
		iFileMenu->AddSeparatorItem();
		
		NewItem(iFileMenu, &iFileQuitItem, "Quit", MID_FILE_QUIT, 'Q');
	
	iMenuBar->AddItem(iFileMenu);

	iEditMenu=new BMenu("Edit");
		NewItem(iEditMenu, &iEditCutItem, "Cut", MID_EDIT_CUT, 'X');
		NewItem(iEditMenu, &iEditCopyItem, "Copy", MID_EDIT_COPY, 'C');
		NewItem(iEditMenu, &iEditPasteItem, "Paste", MID_EDIT_PASTE, 'V');
		NewItem(iEditMenu, &iEditPasteLinkItem, "Paste link", MID_EDIT_PASTE_LINK);
		iEditMenu->AddSeparatorItem();
		
		NewItem(iEditMenu, &iEditSelectAllItem, "Select All", MID_EDIT_SELECT_ALL, 'A');
		NewItem(iEditMenu, &iEditSelectInvertItem, "Invert Selection", MID_EDIT_SELECT_INVERT);
	
	iMenuBar->AddItem(iEditMenu);
	
	iViewMenu=new BMenu("View");
		NewItem(iViewMenu, &iViewRefreshItem, "Refresh", MID_VIEW_REFRESH, 'R');	
		iViewMenu->AddSeparatorItem();
		NewItem(iViewMenu, &iViewColumnsNameItem, "Name", MID_VIEW_COLUMNS_NAME);
		NewItem(iViewMenu, &iViewColumnsSizeItem, "Size", MID_VIEW_COLUMNS_SIZE);
		NewItem(iViewMenu, &iViewColumnsModifiedItem, "Modified", MID_VIEW_COLUMNS_MODIFIED);
		NewItem(iViewMenu, &iViewColumnsCreatedItem, "Created", MID_VIEW_COLUMNS_CREATED);
		NewItem(iViewMenu, &iViewColumnsKindItem, "Kind", MID_VIEW_COLUMNS_KIND);
		
		
	iMenuBar->AddItem(iViewMenu);
	
	/* set initial selections */
	iViewColumnsNameItem->SetMarked(true);
	iViewColumnsSizeItem->SetMarked(true);
	iViewColumnsModifiedItem->SetMarked(true);
		
	AddChild(iMenuBar);
	
	/* build popup menus */
	iVolumePopUpMenu=new BPopUpMenu("Volume", false, false);
		NewItem(iVolumePopUpMenu, &iVolumePopUpOpenItem, "Open", MID_FILE_OPEN, 'O');
		iVolumePopUpMenu->AddSeparatorItem();
		NewItem(iVolumePopUpMenu, &iVolumePopUpEditNameItem, "Rename", MID_FILE_EDIT_NAME, 'E');
		iVolumePopUpMenu->AddSeparatorItem();
		NewItem(iVolumePopUpMenu, &iVolumePopUpTrackerHereItem, "Open in Tracker", MID_TRACKER_HERE);
		NewItem(iVolumePopUpMenu, &iVolumePopUpGetInfoItem, "Get Info", MID_FILE_GET_INFO, 'I');

	iFolderPopUpMenu=new BPopUpMenu("File", false, false);
		NewItem(iFolderPopUpMenu, &iFolderPopUpOpenItem, "Open", MID_FILE_OPEN, 'O');
		iFolderPopUpMenu->AddSeparatorItem();
		NewItem(iFolderPopUpMenu, &iFolderPopUpCutItem, "Cut", MID_EDIT_CUT, 'X');
		NewItem(iFolderPopUpMenu, &iFolderPopUpCopyItem, "Copy", MID_EDIT_COPY, 'C');
		NewItem(iFolderPopUpMenu, &iFolderPopUpPasteItem, "Paste", MID_EDIT_PASTE, 'V');
		NewItem(iFolderPopUpMenu, &iFolderPopUpPasteLinkItem, "Paste Link", MID_EDIT_PASTE_LINK);
		iFolderPopUpMenu->AddSeparatorItem();
		NewItem(iFolderPopUpMenu, &iFolderPopUpEditNameItem, "Rename", MID_FILE_EDIT_NAME, 'E');
		NewItem(iFolderPopUpMenu, &iFolderPopUpMoveToTrashItem, "Send to Trash", MID_FILE_MOVE_TO_TRASH, 'T');
		iFolderPopUpMenu->AddSeparatorItem();
		NewItem(iFolderPopUpMenu, &iFolderPopUpTrackerHereItem, "Open in Tracker", MID_TRACKER_HERE);
		NewItem(iFolderPopUpMenu, &iFolderPopUpGetInfoItem, "Get Info", MID_FILE_GET_INFO, 'I');

	iFilePopUpMenu=new BPopUpMenu("File", false, false);
		NewItem(iFilePopUpMenu, &iFilePopUpOpenItem, "Open", MID_FILE_OPEN, 'O');
		iFilePopUpMenu->AddSeparatorItem();
		NewItem(iFilePopUpMenu, &iFilePopUpCutItem, "Cut", MID_EDIT_CUT, 'X');
		NewItem(iFilePopUpMenu, &iFilePopUpCopyItem, "Copy", MID_EDIT_COPY, 'C');
		iFilePopUpMenu->AddSeparatorItem();
		NewItem(iFilePopUpMenu, &iFilePopUpEditNameItem, "Rename", MID_FILE_EDIT_NAME, 'E');
		NewItem(iFilePopUpMenu, &iFilePopUpMoveToTrashItem, "Send to Trash", MID_FILE_MOVE_TO_TRASH, 'T');
		iFilePopUpMenu->AddSeparatorItem();
		NewItem(iFilePopUpMenu, &iFilePopUpGetInfoItem, "Get Info", MID_FILE_GET_INFO, 'I');

	iTrashPopUpMenu=new BPopUpMenu("File", false, false);
		NewItem(iTrashPopUpMenu, &iTrashPopUpEmptyTrashItem, "Empty Trash", MID_FILE_EMPTY_TRASH);
		iTrashPopUpMenu->AddSeparatorItem();
		NewItem(iTrashPopUpMenu, &iTrashPopUpOpenItem, "Open", MID_FILE_OPEN, 'O');
		NewItem(iTrashPopUpMenu, &iTrashPopUpGetInfoItem, "Get Info", MID_FILE_GET_INFO, 'I');
}

/* create tree-view */
void SeekerWindow::BuildTreeView()
{
	BRect bounds=this->Bounds();
	
	/* a bug in the ColumnListView stuff means that we have to
	   pre-calculate the bounds of the window before creating,
	   otherwise the scrollbars don't show up */
	bounds=iSplitterView->CalcPaneBounds(C_LEFT);
	bounds.bottom -=B_H_SCROLL_BAR_HEIGHT + 4; /* 4=FANCY BORDER WIDTH */
	bounds.right -=B_V_SCROLL_BAR_WIDTH + 4;
	
	iTreeView=new STreeView(bounds, "SeekerWindow::TreeView");
	iSplitterView->AddChild(iTreeView->View(), C_LEFT);
}

/* create file-view */
void SeekerWindow::BuildFileView()
{
	BRect bounds=this->Bounds();

	/* a bug in the ColumnListView stuff means that we have to
	   pre-calculate the bounds of the window before creating,
	   otherwise the scrollbars don't show up */
	bounds=iSplitterView->CalcPaneBounds(C_RIGHT);
	bounds.bottom -=B_H_SCROLL_BAR_HEIGHT + 4; /* 4=FANCY BORDER WIDTH */
	bounds.right -=B_V_SCROLL_BAR_WIDTH + 4;

	iFileView=new SFileView(bounds, "SeekerWindow::FileView");
	iSplitterView->AddChild(iFileView->View(), C_RIGHT);
}

/* toggle column selection and update file-view */
void SeekerWindow::UpdateColumns(BMenuItem *item)
{
	/* invert mark */
	item->SetMarked(!item->IsMarked());

	/* update view */
	iFileView->SetColumns(iViewColumnsNameItem->IsMarked(),
						   iViewColumnsSizeItem->IsMarked(),
						   iViewColumnsModifiedItem->IsMarked(),
						   iViewColumnsCreatedItem->IsMarked(),
						   iViewColumnsKindItem->IsMarked(),
						   false);

}

/* update path in window title */
void SeekerWindow::UpdateTitle(BMessage *message)
{
	if(message==NULL) {
		return;
	}
	
	const char *path;
	if(message->FindString("path", &path) !=B_OK) {
		return;
	}
	
	char strg[B_PATH_NAME_LENGTH+30];
	sprintf(strg, "%s : %s", WINDOW_TITLE, path);

	SetTitle(strg);
}

/* remove floating window from subset */
void SeekerWindow::RemoveFromSubset(BMessage *message)
{
	if(message==NULL) {
		return;
	}
	
	BWindow *window;
	if(message->FindPointer("window", (void **) &window) !=B_OK) {
		return;
	}
	
	BWindow::RemoveFromSubset(window);
}

/* current selections */
void SeekerWindow::CurrentSelections(SFileList *fileList)
{
	if(iTreeView->IsFocus()) {
		iTreeView->CurrentSelections(fileList);
	} else {
		iFileView->CurrentSelections(fileList);		
	
		if(fileList->CountItems()==0) {
			/* use tree-view instead */
			iTreeView->CurrentSelections(fileList);
		}
	}
}

/* current folder */
void SeekerWindow::CurrentFolder(SFile **file, bool useSuperItem)
{
	SFileList fileList;
	iTreeView->CurrentSelections(&fileList);
	
	if(fileList.CountItems() > 0) {	

		/* theoretically, this list should contain one and always one item */	
		*file=new SFile((SFile *) fileList.ItemAt(0));

		/* return parent folder ? */
		/* cut and copy routines have trouble using directories selected from the
		   tree-view.  In addition, this provides protection from copying the Desktop
		   root */
		if(iTreeView->IsFocus()) {
		
			if((*file)->IsDesktop()) {
				delete *file;
				*file=0;
				return;
			}
		
			**file=*((*file)->Parent());

		} 
		
	} else {
		*file=0;
	}
	
}

void SeekerWindow::VolumePopUp(BMessage *message)
{
	if(message==NULL)
		return;

	BPoint point;
	if(message->FindPoint("point", &point) !=B_OK)
		return;

	STreeItem *item;
	if(message->FindPointer("item", (void **) &item) !=B_OK)
		return;

	BMenuItem *result=iVolumePopUpMenu->Go(point);
	
	uint32 msgId=0;	

	if(result==iVolumePopUpOpenItem)
		msgId=MID_FILE_OPEN;
	else if(result==iVolumePopUpGetInfoItem)
		msgId=MID_FILE_GET_INFO;
	else if(result==iVolumePopUpEditNameItem)
		msgId=MID_FILE_EDIT_NAME;
	else if(result==iVolumePopUpTrackerHereItem)
		msgId=MID_TRACKER_HERE;
	
	if(msgId)
	{
		BMessage message(msgId);
		message.AddInt8("fileview",0);
		Looper()->PostMessage(&message);
	}

}

void SeekerWindow::FolderPopUp(BMessage *message)
{
	if(message==NULL)
		return;

	BPoint point;
	if(message->FindPoint("point", &point) !=B_OK)
		return;

	STreeItem *item;
	if(message->FindPointer("item", (void **) &item) !=B_OK)
		return;

	int8 fileview;
	if(message->FindInt8("fileview", &fileview) !=B_OK)
		fileview=0;
	BMenuItem *result=iFolderPopUpMenu->Go(point, true);
	
	uint32 msgId=0;	

	if(result==iFolderPopUpOpenItem) {
		msgId=MID_FILE_OPEN;
	} else if(result==iFolderPopUpGetInfoItem) {
		msgId=MID_FILE_GET_INFO;
	} else if(result==iFolderPopUpEditNameItem) {
		msgId=MID_FILE_EDIT_NAME;
	} else if(result==iFolderPopUpMoveToTrashItem) {
		msgId=MID_FILE_MOVE_TO_TRASH;
	} else if(result==iFolderPopUpCutItem) {
		msgId=MID_EDIT_CUT;
	} else if(result==iFolderPopUpCopyItem) {
		msgId=MID_EDIT_COPY;
	} else if(result==iFolderPopUpPasteItem) {
		msgId=MID_EDIT_PASTE;
	} else if(result==iFolderPopUpPasteLinkItem) {
		msgId=MID_EDIT_PASTE_LINK;
	} else if(result==iFolderPopUpTrackerHereItem)
		msgId=MID_TRACKER_HERE;

	if(msgId)
	{
		BMessage message(msgId);
		message.AddInt8("fileview",fileview);
		Looper()->PostMessage(&message);
	}
}

void SeekerWindow::FilePopUp(BMessage *message)
{
	if(message==NULL)
		return;

	BPoint point;
	if(message->FindPoint("point", &point) !=B_OK)
		return;

	STreeItem *item;
	if(message->FindPointer("item", (void **) &item) !=B_OK)
		return;

	BMenuItem *result=iFilePopUpMenu->Go(point, true);
	
	uint32 msgId=0;	

	if(result==iFilePopUpOpenItem) {
		msgId=MID_FILE_OPEN;
	} else if(result==iFilePopUpGetInfoItem) {
		msgId=MID_FILE_GET_INFO;
	} else if(result==iFilePopUpEditNameItem) {
		msgId=MID_FILE_EDIT_NAME;
	} else if(result==iFilePopUpMoveToTrashItem) {
		msgId=MID_FILE_MOVE_TO_TRASH;
	} else if(result==iFilePopUpCutItem) {
		msgId=MID_EDIT_CUT;
	} else if(result==iFilePopUpCopyItem) {
		msgId=MID_EDIT_COPY;
	}

	if(msgId)
	{
		BMessage message(msgId);
		Looper()->PostMessage(&message);
	}
}

void SeekerWindow::TrashPopUp(BMessage *message)
{
	if(message==NULL)
		return;

	BPoint point;
	if(message->FindPoint("point", &point) !=B_OK)
		return;

	STreeItem *item;
	if(message->FindPointer("item", (void **) &item) !=B_OK)
		return;

	BMenuItem *result=iTrashPopUpMenu->Go(point);
	
	uint32 msgId=0;	

	if(result==iTrashPopUpOpenItem) {
		msgId=MID_FILE_OPEN;
	} else if(result==iTrashPopUpGetInfoItem) {
		msgId=MID_FILE_GET_INFO;
	} else if(result==iTrashPopUpEmptyTrashItem) {
		msgId=MID_FILE_EMPTY_TRASH;
	}
	
	if(msgId)
	{
		BMessage message(msgId);
		Looper()->PostMessage(&message);
	}

}

bool SeekerWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

void SeekerWindow::MessageReceived(BMessage* message)
{
	SFile *currentFolder=0;
	SFileList fileList;

	switch(message->what)
	{
		case MID_FILE_NEW_FOLDER		:
			CurrentSelections(&fileList);
			if(fileList.CountItems() > 0) {
				iFileWorker->NewFolder((SFile *) fileList.ItemAt(fileList.CountItems()-1));
			}
			break;
	
  		case MID_FILE_OPEN				:
			if(iTreeView->IsFocus()) {
		
				/* search tree-view contents for selected item */
				for (int32 i=0; i < iTreeView->CountItems(); i++) {
					STreeItem *item=(STreeItem *) iTreeView->ItemAt(i); 
					if(item->IsSelected()) {
						iTreeView->Expand(item);				
						break;
					}
				}
		
			} else if(iFileView->IsFocus()) {
		
				/* search file-view contents for selected items */
				for (int32 i=0; i < iFileView->CountItems(); i++) {
					SFileItem *item=(SFileItem *) iFileView->ItemAt(i); 
					if(item->IsSelected()) {
						BMessage message(MSG_FILEVIEW_INVOKE);
						message.AddInt32("index", iFileView->IndexOf(item));
						Looper()->PostMessage(&message);
						break;
					}
				}
			}
  			break;
  			
  		case MID_FILE_GET_INFO			:
			CurrentSelections(&fileList);
			if(fileList.CountItems() > 0) {
  				iFileWorker->GetInfo((SFile *) fileList.ItemAt(fileList.CountItems()-1));
			}
			break;
			
		case MID_FILE_EDIT_NAME			:
			CurrentSelections(&fileList);
			if(fileList.CountItems() > 0) {
				iFileWorker->EditName((SFile *) fileList.ItemAt(fileList.CountItems()-1));
			}
			break;
		
		case MID_FILE_MOVE_TO_TRASH		:
			CurrentSelections(&fileList);
			CurrentFolder(&currentFolder, true);
			if(currentFolder) {
				iFileWorker->MoveToTrash(currentFolder, &fileList);
			}
			break;
			
		case MID_FILE_EMPTY_TRASH		:
			iFileWorker->EmptyTrash();
			break;

 		case MID_FILE_QUIT				:
			Quit();
  			break;

		case MID_EDIT_CUT				:
			CurrentSelections(&fileList);
			CurrentFolder(&currentFolder, true);
			if(currentFolder) {
				iFileWorker->Cut(currentFolder, &fileList);
			}
			break;
		
		case MID_EDIT_COPY				:
			CurrentSelections(&fileList);
			CurrentFolder(&currentFolder, true);
			if(currentFolder) {
				iFileWorker->Copy(currentFolder, &fileList);
			}
			break;
			
		case MID_EDIT_PASTE				:
			CurrentSelections(&fileList);
			if(fileList.CountItems() > 0) {
				iFileWorker->Paste((SFile *) fileList.ItemAt(fileList.CountItems()-1));
			}
			break;
		
		case MID_EDIT_PASTE_LINK		:
			CurrentSelections(&fileList);
			if(fileList.CountItems() > 0) {
				iFileWorker->PasteLink((SFile *) fileList.ItemAt(fileList.CountItems()-1));
			}
			break;
			  			
		case MID_EDIT_SELECT_ALL		:
			iFileView->SelectAll(); 
			break;
			
		case MID_EDIT_SELECT_INVERT		:
			iFileView->InvertSelection();
			break;

  		case MID_VIEW_COLUMNS_NAME 		:
			UpdateColumns(iViewColumnsNameItem);
			break;

  		case MID_VIEW_COLUMNS_SIZE 		:
			UpdateColumns(iViewColumnsSizeItem);
			break;

		case MID_VIEW_COLUMNS_MODIFIED	:
			UpdateColumns(iViewColumnsModifiedItem);
			break;

		case MID_VIEW_COLUMNS_CREATED	:
			UpdateColumns(iViewColumnsCreatedItem);
			break;

		case MID_VIEW_COLUMNS_KIND		:
			UpdateColumns(iViewColumnsKindItem);
			break;

  		case MID_VIEW_REFRESH			:
  		case B_DEVICE_MOUNTED:
  		case B_DEVICE_UNMOUNTED:
  			iTreeView->Refresh();
  			break;

		case MID_TRACKER_HERE:
		{
			int8 fileview;
			if(message->FindInt8("fileview",&fileview)==B_OK)
			{
				if(fileview)
					iFileView->SpawnTracker();
				else
					iTreeView->SpawnTracker();
			}
			break;
		}
  		case MSG_PIONEER_UPDATE_TITLE		:
  			UpdateTitle(message);
  			break;
  		
  		case MSG_PIONEER_REMOVE_FROM_SUBSET :
  			RemoveFromSubset(message); 
  			break;
  			
  		case MSG_PIONEER_VOLUME_POPUP_MENU 	:
  			VolumePopUp(message);
  			break;

		case MSG_PIONEER_FOLDER_POPUP_MENU	:
			FolderPopUp(message);
			break;
  			
  		case MSG_PIONEER_FILE_POPUP_MENU	:
  			FilePopUp(message);
  			break;
  			
  		case MSG_PIONEER_TRASH_POPUP_MENU	:
  			TrashPopUp(message);
  			break;
  		
  		case MSG_TREEVIEW_EXPAND		:
  			iTreeView->ExpandAndSelect(message);
  			break;
  		
  		case MSG_FILEVIEW_REFRESH		:
 			iFileView->Refresh(message);
  			break;
  		
  		case MSG_FILEVIEW_INVOKE		:
  			iFileView->Open(message);
  			break;
  		
  		case MSG_FILEVIEW_CLEAR			:
  			iFileView->Clear();
  			break;
  			
  		case MSG_NODEMONITOR_ADD_WATCH	:
  			iNodeMonitor->AddWatch(message);
  			break;
  			
  		case MSG_NODEMONITOR_REMOVE_WATCH :
  			iNodeMonitor->RemoveWatch(message);
  			break;
  			
		default:
			BWindow::MessageReceived(message);
	}
}

